#####choose Environment data
library(rgdal)
library(raster)

randompoint<-readOGR("E:/GIS Data/1000_random_points.shp")
env.final<-list.files("E:/Env_layer_GCS_1km_clip",pattern = ".tif",full.names = TRUE)

for(i in 1:24)
{
  temp<-raster(env.final[i])
  print(names(temp))
  if(i == 1)
    env.stack <- stack(temp)
  else env.stack <- stack(env.stack,temp)
}


env.point <- extract(env.stack, randompoint, method='simple')
new.env<-as.data.frame(env.point)


new.env<- new.env[complete.cases(new.env),]

cor_value <- cor(new.env,method="spearman")
View(cor_value)

cor_value <-round(cor_value,2)

env.cor<-as.data.frame(cor_value )
write.csv(env.cor,"E:/1000randompoint_24env_cor.csv")

#####modelling
#data you collect
#thin
loc <- read.csv("location data.csv")
absence <- loc[c(1:49),]
presence <- loc[c(50:104),]
thin(
  presence,
  lat.col = "lat",
  long.col = "lon",
  spec.col = "No.",30,1, 
  locs.thinned.list.return = FALSE,
  write.files = TRUE,
  max.files = 1400,
  "E:/location thin data.csv")


species <- read.csv("E:/location thin data.csv",header=T)
#varibles you collect
env.final<-list.files("E:/Env_layer_GCS_30km_select9",pattern = ".tif",full.names = T)

for(i in 1:9){
  temp<-raster(env.final[i])
  print(names(temp))
  if(i==1)
    env.stack <- stack(temp)
  else env.stack <- stack(env.stack,temp)
}

#env.stack[[9]]=as.factor(env.stack[[9]])
#modeling data
library(raster)
library(biomod2)

species.data = 
  BIOMOD_FormatingData(
    resp.var = as.numeric(species[,4]),
    expl.var = env.stack,
    resp.xy = species[, 2:3],
    resp.name = 'CM0316'
  )
set.seed(20220517)
#modeling
SDM.30km <-
  BIOMOD_Modeling(
    data = species.data,
    models = 'RF',
    NbRunEval = 3,
    DataSplit= 80,
    VarImport=3,
    models.eval.meth = c('KAPPA','TSS','ROC') ,
    SaveObj = T
  )

SDM.30km.eval<-get_evaluations(SDM.30km)
SDM.30km.eval["ROC","Testing.data","RF",,]
SDM.30km.eval["KAPPA","Cutoff","RF",,]

get_variables_importance(SDM.10km)

#project
preLow.final = BIOMOD_Projection(SDM.30km,env.stack, '30km')
lowRpredict=preLow.final@proj@val[[2]] 

#result and plot
plot(lowRpredict)
writeRaster(lowRpredict,"E:/9var_pre48_absen49_30km.tif")
cutoff <- lowRpredict>705.5
plot(cutoff)
writeRaster(cutoff,"E:/RF_model_30km.tif")


#####ensemble model
myBiomodModelOut <-
  BIOMOD_Modeling(
    data = species.data,
    models = c("GLM", "GBM","RF"),
    NbRunEval = 1,
    DataSplit= 80,
    VarImport=3,
    models.eval.meth = c('KAPPA','TSS','ROC'),
    SaveObj = T
  )

get_evaluations(myBiomodModelOut)
get_built_models(myBiomodModelOut)
# Doing Ensemble Modelling
myBiomodEM <- 
  BIOMOD_EnsembleModeling( modeling.output = myBiomodModelOut,
                           chosen.models = 'all',
                           em.by = 'all',
                           eval.metric = c('ROC'),
                           eval.metric.quality.threshold = c(0.9),
                           models.eval.meth = c('KAPPA','TSS','ROC'),
                           prob.mean = TRUE,
                           prob.cv = FALSE,
                           prob.ci = FALSE,
                           prob.ci.alpha = 0.05,
                           prob.median = FALSE,
                           committee.averaging = FALSE,
                           prob.mean.weight = TRUE,
                           prob.mean.weight.decay = 'proportional')
# print summary
myBiomodEM
# get evaluation scores
get_evaluations(myBiomodEM)
get_evaluations(myBiomodModelOut)
get_built_models(myBiomodEM)
get_variables_importance(myBiomodModelOut)
myBiomodEM
# Individual models projections on current environmental conditions
myBiomodProjection <- BIOMOD_Projection(
  modeling.output = myBiomodModelOut,
  new.env =  env.stack,
  proj.name = 'current',
  selected.models = 'all',
  compress = FALSE,
  build.clamping.mask = FALSE)
#  Creating the ensemble projections
EnsembleForecasting <- BIOMOD_EnsembleForecasting( 
  projection.output = myBiomodProjection,
  EM.output = myBiomodEM)
predict <- EnsembleForecasting@proj@val[[2]] 

#result and plot
plot(predict)
writeRaster(predict,"E:/9var_pre48_absen49_30km_ensemble.tif")
##kappa cut off
cutoff <- predict>491.5
plot(cutoff)
writeRaster(cutoff,"E:/9var_pre48_absen49_30km_ensemble_cut491.5.tif")


response_curve <- biomod2::response.plot2(
  models = BIOMOD_LoadModels(myBiomodModelOut),
  Data = get_formal_data(myBiomodModelOut,"expl.var"),
  show.variables = get_formal_data(myBiomodModelOut,"expl.var.names"),
  do.bivariate = FALSE,
  fixed.var.metric = "mean",
  col = c("blue","green","purple","red","yellow","black"),
  legend = TRUE
)

title(ylab="Probability of presence")
